#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define VENDOR_RQ_SET_LED    0
#define VENDOR_RQ_GET_SWITCH 1
#define VENDOR_RQ_GET_SWITCH2 2
#define VENDOR_RQ_GET_SWITCH3 3
#define VENDOR_RQ_GET_SWITCH4 4
/* ------------------------------------------------------------------------- */
/* ---------------------------- Utility Functions -------------------------- */
/* ------------------------------------------------------------------------- */

uint8_t in_c(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRC &= ~(1<<pin);
    return (PINC >>pin) & 1;
} 
uint8_t in_b(uint8_t pin)
{
    // read logic level of port-C's pin
    DDRB &= ~(1<<pin);
    return (PINB >>pin) & 1;
} 
void out_c(uint8_t pin, uint8_t val)
{
    // write logic level to port-C's pin
    DDRC |= 1<<pin;
    if (val)
        PORTC |= 1<<pin;
    else
        PORTC &= ~(1<<pin);
}
void out_b(uint8_t pin, uint8_t val)
{
    // write logic level to port-B's pin
    DDRB |= 1<<pin;
    if (val)
        PORTB |= 1<<pin;
    else
        PORTB &= ~(1<<pin);
}
/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t dataBuffer[1];  /* must stay valid when usbFunctionSetup returns */

    if (rq->bRequest == VENDOR_RQ_SET_LED)
    {
        uint8_t led_no  = rq->wValue.bytes[0];
        uint8_t led_val = rq->wValue.bytes[1];
        out_b(led_no, led_val);

        return 0;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH)
    {
        dataBuffer[0] = !in_c(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH2)
    {
        dataBuffer[0] = !in_c(5);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH3)
    {
        dataBuffer[0] = !in_b(5);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    else if (rq->bRequest == VENDOR_RQ_GET_SWITCH4)
    {
        dataBuffer[0] = !in_b(3);
        usbMsgPtr = dataBuffer;
        return 1;
    }
    return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
    DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
    DDRC  = (1<<PB0)|(1<<PB1)|(1<<PB2); // PC0..PC2 = Output, PC3..PC5 = Input
    PORTC = (1 << PC3) | (1 << PC5);                 // Enable pull-up at PC3
    PORTB = (1 << PB3) | (1 << PB5); 

    usbInit();
    usbDeviceDisconnect();  
    _delay_ms(300);
    usbDeviceConnect();
    sei();
    for(;;){                
        usbPoll();
    }
    return 0;

/*while(1){
    DDRB |= 0b00010001;
    PORTB = 0b00000001;
    PORTB &= 0b11101111;
}*/

}

/* ------------------------------------------------------------------------- */
